@@ -29,22 +29,131 @@ class EnumSolution(object):
         self.objective_value = objective_value
 
 
-def create_enum_variables(model, reaction_weights, eps=1e-2, thr=1e-5, full=False):
-    for rid in reaction_weights.keys():
-        if reaction_weights[rid] == 0:
-            pass
-        elif full and 'x_'+rid not in model.solver.variables:
-            model = create_full_variables(model=model, reaction_weights=reaction_weights, epsilon=eps, threshold=thr)
-            break
-        elif not full and 'rh_'+rid+'_pos' not in model.solver.variables and 'rl_'+rid not in model.solver.variables:
-            model = create_new_partial_variables(model=model, reaction_weights=reaction_weights, epsilon=eps,
-                                                 threshold=thr)
-            break
+def create_new_partial_variable_single(model, rid, epsilon, threshold, pos):
+    if pos and 'rh_' + rid + '_pos' not in model.solver.variables:
+        rxn = model.reactions.get_by_id(rid)
+        xtot = model.solver.interface.Variable('x_%s' % rid, type='binary')
+        xf = model.solver.interface.Variable('rh_%s_pos' % rid, type='binary')
+        xr = model.solver.interface.Variable('rh_%s_neg' % rid, type='binary')
+        model.solver.add(xtot)
+        model.solver.add(xf)
+        model.solver.add(xr)
+        xtot_def = model.solver.interface.Constraint(xtot - xf - xr, lb=0., ub=0., name='x_%s_def' % rid)
+        xf_upper = model.solver.interface.Constraint(
+            rxn.forward_variable - rxn.upper_bound * xf, ub=0., name='xr_%s_upper' % rid)
+        xr_upper = model.solver.interface.Constraint(
+            rxn.reverse_variable + rxn.lower_bound * xr, ub=0., name='xf_%s_upper' % rid)
+        xf_lower = model.solver.interface.Constraint(
+            rxn.forward_variable - epsilon * xf, lb=0., name='xf_%s_lower' % rid)
+        xr_lower = model.solver.interface.Constraint(
+            rxn.reverse_variable - epsilon * xr, lb=0., name='xr_%s_lower' % rid)
+        model.solver.add(xtot_def)
+        model.solver.add(xf_upper)
+        model.solver.add(xr_upper)
+        model.solver.add(xf_lower)
+        model.solver.add(xr_lower)
+    if not pos and 'rl_' + rid not in model.solver.variables:
+        rxn = model.reactions.get_by_id(rid)
+        xtot = model.solver.interface.Variable('rl_%s' % rid, type='binary')
+        xf = model.solver.interface.Variable('xf_%s' % rid, type='binary')
+        xr = model.solver.interface.Variable('xr_%s' % rid, type='binary')
+        model.solver.add(xtot)
+        model.solver.add(xf)
+        model.solver.add(xr)
+        xtot_def = model.solver.interface.Constraint(xtot - xf - xr, lb=0., ub=0., name='x_%s_def' % rid)
+        xf_upper = model.solver.interface.Constraint(
+            rxn.forward_variable - rxn.upper_bound * xf, ub=0., name='xr_%s_upper' % rid)
+        xr_upper = model.solver.interface.Constraint(
+            rxn.reverse_variable + rxn.lower_bound * xr, ub=0., name='xf_%s_upper' % rid)
+        xf_lower = model.solver.interface.Constraint(
+            rxn.forward_variable - threshold * xf, lb=0., name='xf_%s_lower' % rid)
+        xr_lower = model.solver.interface.Constraint(
+            rxn.reverse_variable - threshold * xr, lb=0., name='xr_%s_lower' % rid)
+        model.solver.add(xtot_def)
+        model.solver.add(xf_upper)
+        model.solver.add(xr_upper)
+        model.solver.add(xf_lower)
+        model.solver.add(xr_lower)
     return model
 
 
+def create_full_variable_single(model, rid, reaction_weights, epsilon, threshold):
+    if 'x_' + rid not in model.solver.variables:
+        rxn = model.reactions.get_by_id(rid)
+        xtot = model.solver.interface.Variable('x_%s' % rid, type='binary')
+        xf = model.solver.interface.Variable('xf_%s' % rid, type='binary')
+        xr = model.solver.interface.Variable('xr_%s' % rid, type='binary')
+        model.solver.add(xtot)
+        model.solver.add(xf)
+        model.solver.add(xr)
+        xtot_def = model.solver.interface.Constraint(xtot - xf - xr, lb=0., ub=0., name='x_%s_def' % rid)
+        xf_upper = model.solver.interface.Constraint(
+            rxn.forward_variable - rxn.upper_bound * xf, ub=0., name='xr_%s_upper' % rid)
+        xr_upper = model.solver.interface.Constraint(
+            rxn.reverse_variable + rxn.lower_bound * xr, ub=0., name='xf_%s_upper' % rid)
+        temp = threshold
+        if rid in reaction_weights:
+            if reaction_weights[rid] > 0.:
+                temp = epsilon
+        xf_lower = model.solver.interface.Constraint(
+            rxn.forward_variable - temp * xf, lb=0., name='xf_%s_lower' % rid)
+        xr_lower = model.solver.interface.Constraint(
+            rxn.reverse_variable - temp * xr, lb=0., name='xr_%s_lower' % rid)
+        model.solver.add(xtot_def)
+        model.solver.add(xf_upper)
+        model.solver.add(xr_upper)
+        model.solver.add(xf_lower)
+        model.solver.add(xr_lower)
+    return model
+
+
+def create_enum_variables(model, reaction_weights, eps=1e-2, thr=1e-5, full=False):
+    for rid in reaction_weights.keys():
+        if rid not in model.reactions:
+            pass
+        elif full and 'x_' + rid not in model.solver.variables:
+            model = create_full_variable_single(model=model, rid=rid, reaction_weights=reaction_weights, epsilon=eps,
+                                                threshold=thr)
+        elif reaction_weights[rid] > 0 and 'rh_' + rid + '_pos' not in model.solver.variables:
+            model = create_new_partial_variable_single(model=model, epsilon=eps, threshold=thr, rid=rid, pos=True)
+        elif reaction_weights[rid] < 0 and 'rl_' + rid not in model.solver.variables:
+            model = create_new_partial_variable_single(model=model, epsilon=eps, threshold=thr, rid=rid, pos=False)
+    return model
+
+
+# def create_enum_variables(model, reaction_weights, eps=1e-2, thr=1e-5, full=False):
+#     for rid in reaction_weights.keys():
+#         if full and 'x_'+rid not in model.solver.variables:
+#             model = create_full_variables(model=model, reaction_weights=reaction_weights, epsilon=eps, threshold=thr)
+#             break
+#         elif reaction_weights[rid] == 0:
+#             pass
+#         elif not full and 'rh_'+rid+'_pos' not in model.solver.variables and 'rl_'+rid not in model.solver.variables:
+#             model = create_new_partial_variables(model=model, reaction_weights=reaction_weights, epsilon=eps,
+#                                                  threshold=thr)
+#             break
+#     return model
+
+
 def get_recent_solution_and_iteration(dirpath, startsol_num):
-    paths = sorted(list(Path(dirpath).glob('*solution_*.csv')), key=os.path.getctime)
+    """
+    This functions fetches a solution from a given directory. The solutions are ordered by creation time, and one
+    solution is picked using an exponential distribution (meaning that the most recent solution has the highest
+    probability of being chosen)
+
+    Parameters
+    ----------
+    dirpath: str
+        a directory containing imat or enumeration solutions
+    startsol_num: int
+        the number of starting solutions present in the directory
+
+    Returns
+    -------
+    solution: a Solution object
+    iteration: an int which
+    """
+    paths = sorted(list(Path(dirpath).glob('*solution*.csv')), key=os.path.getctime)
     paths.reverse()
     solpath = paths[int(np.random.exponential(5))]
     solution, binary = read_solution(solpath)
@@ -54,6 +163,9 @@ def get_recent_solution_and_iteration(dirpath, startsol_num):
 
 def write_rxn_enum_script(directory, modelfile, weightfile, imatsol=None, reactionlist=None, eps=1e-4, thr=1e-5,
                           tol=1e-8, iters=100, maxiters=1e10):
+    os.makedirs(directory, exist_ok=True)
+    if directory[-1] not in ['/', '\\']:
+        directory += '/'
     if reactionlist is not None:
         with open(reactionlist, 'r') as file:
             rxns = file.read().split('\n')
@@ -87,6 +199,9 @@ def write_rxn_enum_script(directory, modelfile, weightfile, imatsol=None, reacti
 
 def write_batch_script_divenum(directory, modelfile, weightfile, rxnsols, objtol, eps=1e-4, thr=1e-5,
                                tol=1e-8, filenums=100, iters=100, t=6000):
+    os.makedirs(directory, exist_ok=True)
+    if directory[-1] not in ['/', '\\']:
+        directory += '/'
     for i in range(filenums):
         with open(directory+'file_'+str(i)+'.sh', 'w+') as f:
             f.write('#!/bin/bash\n#SBATCH -p workq\n#SBATCH --mail-type=ALL\n#SBATCH --mem=64G\n#SBATCH -c 24\n'
@@ -116,7 +231,7 @@ def write_batch_script1(directory, modelfile, weightfile, cplexpath, reactionlis
     Parameters
     ----------
     directory: str
-        directory in which the files will be generated. MUST have a '/' and be a valid directory
+        directory in which the files will be generated. If it does not exist, it will be created
     modelfile: str
         path to the model
     weightfile:
@@ -134,6 +249,9 @@ def write_batch_script1(directory, modelfile, weightfile, cplexpath, reactionlis
     iters: int
         number of diversity-enumeration iterations per batch
     """
+    os.makedirs(directory, exist_ok=True)
+    if directory[-1] not in ['/', '\\']:
+        directory += '/'
     if reactionlist is not None:
         rstring = '-l ' + reactionlist
     else:
@@ -183,6 +301,9 @@ def write_batch_script2(directory, modelfile, weightfile, cplexpath, objtol=1e-2
     filenums: int
         number of parallel batches
     """
+    os.makedirs(directory, exist_ok=True)
+    if directory[-1] not in ['/', '\\']:
+        directory += '/'
     paths = sorted(list(Path(directory).glob('*solution_*.csv')), key=os.path.getctime)
     paths.reverse()
     for i in range(filenums):
