@@ -1,10 +1,11 @@
+import argparse
 import numpy as np
 import pandas as pd
-from pathlib import Path
-from cobra import Solution
 import matplotlib.pyplot as plt
+from pathlib import Path
+from warnings import warn
+from cobra import Solution
 from sklearn.decomposition import PCA
-import argparse
 
 
 def write_solution(model, solution, threshold, filename='imat_sol.csv'):
@@ -18,7 +19,6 @@ def write_solution(model, solution, threshold, filename='imat_sol.csv'):
     """
     tol = model.solver.configuration.tolerances.feasibility
     solution_binary = (np.abs(solution.fluxes) >= threshold-tol).values.astype(int)
-
     with open(filename, 'w+') as file:
         file.write('reaction,fluxes,binary\n')
         for i, v in enumerate(solution.fluxes):
@@ -39,9 +39,13 @@ def read_solution(filename, model=None):
             objective_value = float(reader[-2].split()[-1])
             status = reader[-1].split()[-1]
     if binary:
-        fluxes = pd.read_csv(filename, index_col=0).rename(index={0: 'fluxes'}).T
-        fluxes.index = [rxn.id for rxn in model.reactions]
-        sol_bin = list(fluxes['fluxes'])
+        fluxes = pd.read_csv(filename, index_col=0).iloc[0]
+        if model is not None:
+            fluxes.index = [rxn.id for rxn in model.reactions]
+        else:
+            warn('A model is necessary for setting the reaction IDs in a binary solution.'
+                 'Disregard this warning if the columns of the binary solution are already reaction IDs')
+        sol_bin = np.array(fluxes.values)
         objective_value = 0.
         status = 'binary'
     else:
@@ -52,7 +56,7 @@ def read_solution(filename, model=None):
     return solution, sol_bin
 
 
-def combine_solutions(sol_path):
+def combine_binary_solutions(sol_path):
     solutions = Path(sol_path).glob('*solutions.csv')
     sollist = []
     for sol in solutions:
@@ -60,10 +64,58 @@ def combine_solutions(sol_path):
     fullsol = pd.concat(sollist, ignore_index=True)
     uniquesol = fullsol.drop_duplicates()
     print('There are %i unique solutions and %i duplicates.' % (len(uniquesol), len(fullsol) - len(uniquesol)))
-    uniquesol.to_csv(sol_path+'/combined_solutions.csv')
+    uniquesol.to_csv(sol_path+'combined_solutions.csv')
     return uniquesol
 
 
+def compile_solutions(solutions, out_path='compiled_solutions', model=None, threshold=None):
+    """
+    Compiles individual solution files into one binary solution DataFrame
+    Parameters
+    ----------
+    solutions: list or str
+        if list: either a list of solution files in .csv format
+                 or a list of Solution objects
+                 or a list of binary solution arrays
+        if str: a folder in which all .csv files are solutions
+    out_path: str
+        path to which the combined solutions will be saved
+    model: cobrapy Model
+        necessary if the solutions parameter is a list of Solution objects
+    threshold: float
+        required if the solutions parameter is a list of Solution objects
+    Returns
+    -------
+    sol_frame: pandas DataFrame containg binary solutions
+    """
+    if model is not None:
+        tol = model.solver.configuration.tolerances.feasibility
+    if isinstance(solutions, str):
+        sol_paths = [str(x) for x in Path(solutions).glob('*.csv')]
+    else:
+        sol_paths = solutions
+    sols = []
+    for s in sol_paths:
+        binsol = None
+        if isinstance(s, str):
+            fullsol, binsol = read_solution(s, model=model)
+        elif isinstance(s, Solution):
+            try:
+                binsol = (np.abs(s.fluxes) >= threshold - tol).values.astype(int)
+            except TypeError:
+                warn('If you pass a list of Solution objects, you must also provide the model and threshold parameters.'
+                     'The current model parameter is %s and the current threshold parameter is %s' % (model, threshold))
+        elif isinstance(s, list) or isinstance(s, np.ndarray):
+            binsol = np.array(s)
+        else:
+            warn('Unrecognized type %s for solution %s' % (type(s), s))
+        if binsol is not None:
+            sols.append(binsol)
+    sol_frame = pd.DataFrame(sols)
+    sol_frame.to_csv(out_path+'.csv')
+    return sol_frame
+
+
 def plot_pca(solution_path, rxn_enum_solutions=None, save=True, save_name=''):
     """
     Plots a 2-dimensional PCA of enumeration solutions
@@ -117,7 +169,6 @@ def plot_pca(solution_path, rxn_enum_solutions=None, save=True, save_name=''):
     plt.legend(fontsize='large')
     if save:
         fig.savefig(save_name+'pca.png')
-
     return pca
 
 
